// SPDX-License-Identifier: GPL-2.0-only OR MIT
/*

Copyright (C) 2023 The Falco Authors.

This file is dual licensed under either the MIT or GPL 2. See MIT.txt
or GPL2.txt for full copies of the license.

*/
#pragma once

/* These numbers must be updated when we add new events in the event table */
#define SYSCALL_EVENTS_NUM 382
#define TRACEPOINT_EVENTS_NUM 6
#define METAEVENTS_NUM 18
#define PLUGIN_EVENTS_NUM 1
#define UNKNOWN_EVENTS_NUM 23
